﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using IPCInterface;

namespace WinSource {
	[ServiceBehavior(ConcurrencyMode = ConcurrencyMode.Reentrant, InstanceContextMode = InstanceContextMode.Single)]
	public partial class MainWindow : Window, IS2CMessages {
		public bool Status { get; set; }

		Guid _clientID;
		ServiceHost _clientHost;

		public MainWindow() {
			InitializeComponent();

			(PresentationSource.FromVisual(this) as HwndSource).AddHook(new HwndSourceHook(this.WndProc));

			Status = false;

			_clientID = Guid.NewGuid();
			_clientHost = new ServiceHost(this);

			_clientHost.AddServiceEndpoint((typeof(IS2CMessages)), new NetNamedPipeBinding(), "net.pipe://localhost/Client_" + _clientID.ToString());
			_clientHost.Open();

			this.Loaded += OnLoaded;
		}

		void OnLoaded(object sender, RoutedEventArgs e) {
			this.Dispatcher.BeginInvoke(new Action(() => {
				Register(_clientID);
			}));

			media.LoadedBehavior = MediaState.Manual;
			media.Source = new Uri(@"D:\Test\IMG_0245.MOV");
			media.Play();
		}


		public void Register(Guid clientID) {
			using (ChannelFactory<IC2SMessages> factory = new ChannelFactory<IC2SMessages>(new NetNamedPipeBinding(), new EndpointAddress("net.pipe://localhost/Server"))) {
				IC2SMessages clientToServerChannel = factory.CreateChannel();
				try {
					clientToServerChannel.Register(clientID);
					clientToServerChannel.DisplayCommandOnServer(string.Format("+Client:{0}", clientID.ToString()));
				} catch (Exception ex) {
					MessageBox.Show(ex.ToString());
				} finally {
					CloseChannel((ICommunicationObject)clientToServerChannel);
				}
			}
		}

		private void CloseChannel(ICommunicationObject channel) {
			try {
				channel.Close();
			} catch (Exception ex) {
				MessageBox.Show(ex.ToString());
			} finally {
				channel.Abort();
			}
		}

		public void CommandInClient(string text) {
			if (text == "Play") {
				media.Play();
			} else {
				media.Pause();
			}
		}

		IntPtr WndProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled) {
			if (msg == MessageHelper.WM_COPYDATA) {
				CopyDataStruct cds = (CopyDataStruct)System.Runtime.InteropServices.Marshal.PtrToStructure(lParam, typeof(CopyDataStruct));
				if (cds.lpData == "Play") {
					if (Status) {
						media.Pause();
						Status = false;
					} else {
						media.Play();
						Status = true;
					}
				}
			}
			return hwnd;
		}
	}

	public class MessageHelper {
		public const int WM_COPYDATA = 0x004A;
	}


	[StructLayout(LayoutKind.Sequential)]
	public struct CopyDataStruct {

		public IntPtr dwData;

		public int cbData;//字符串长度

		[MarshalAs(UnmanagedType.LPStr)]
		public string lpData;//字符串
	}
}
