﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using IPCInterface;

namespace WinTarget {
	[ServiceBehavior(ConcurrencyMode = ConcurrencyMode.Reentrant, InstanceContextMode = InstanceContextMode.Single)]
	public partial class MainWindow : Window, IC2SMessages {
		ServiceHost _serverHost;
		List<Guid> _registeredClients = new List<Guid>();

		public MainWindow() {
			InitializeComponent();

			_serverHost = new ServiceHost(this);

			_serverHost.AddServiceEndpoint((typeof(IC2SMessages)), new NetNamedPipeBinding(), "net.pipe://localhost/Server");
			_serverHost.Open();
		}

		private void SendText(Guid client, string text) {
			using (ChannelFactory<IS2CMessages> factory = new ChannelFactory<IS2CMessages>(new NetNamedPipeBinding(), new EndpointAddress("net.pipe://localhost/Client_" + client.ToString()))) {
				IS2CMessages serverToClientChannel = factory.CreateChannel();
				try {
					serverToClientChannel.CommandInClient(text);
				} catch (Exception ex) {
					MessageBox.Show(ex.ToString());
				} finally {
					CloseChannel((ICommunicationObject)serverToClientChannel);
				}
			}
		}

		private void CloseChannel(ICommunicationObject channel) {
			try {
				channel.Close();
			} catch (Exception ex) {
				MessageBox.Show(ex.ToString());
			} finally {
				channel.Abort();
			}
		}

		public void Register(Guid clientID) {
			if (!_registeredClients.Contains(clientID)) {
				_registeredClients.Add(clientID);				
			}
		}

		public void DisplayCommandOnServer(string text) {
			lbmessage.Content = text;
		}

		private void Play_Click(object sender, RoutedEventArgs e) {
			//MessageHelper.SendMessageByProcess("WinSource", "Play");
			foreach (Guid client in _registeredClients) {
				SendText(client, "Play");
			}
		}

		private void Pause_Click(object sender, RoutedEventArgs e) {
			//MessageHelper.SendMessageByProcess("WinSource", "Play");
			foreach (Guid client in _registeredClients) {
				SendText(client, "Pause");
			}
		}
	}

	public class MessageHelper {
		public const int WM_COPYDATA = 0x004A;

		[DllImport("User32.dll", EntryPoint = "SendMessage")]
		private static extern int SendMessage
		(
		IntPtr hWnd,                   //目标窗体句柄
		int Msg,                       //WM_COPYDATA
		int wParam,                    //自定义数值
		ref  CopyDataStruct lParam     //结构体
		);

		[DllImport("User32.dll", EntryPoint = "FindWindow")]
		public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

		public static void SendMessage(string windowName, string strMsg) {

			if (strMsg == null) return;

			IntPtr hwnd = FindWindow(null, windowName);

			if (hwnd != IntPtr.Zero) {
				CopyDataStruct cds;

				cds.dwData = IntPtr.Zero;
				cds.lpData = strMsg;

				cds.cbData = System.Text.Encoding.Default.GetBytes(strMsg).Length + 1;

				int fromWindowHandler = 0;
				SendMessage(hwnd, WM_COPYDATA, fromWindowHandler, ref  cds);
			}
		}

		public static void SendMessageByProcess(string processName, string strMsg) {
			if (strMsg == null) return;
			var process = Process.GetProcessesByName(processName);
			if (process.FirstOrDefault() == null) return;
			var hwnd = process.FirstOrDefault().MainWindowHandle;
			if (hwnd == IntPtr.Zero) return;

			if (hwnd != IntPtr.Zero) {
				CopyDataStruct cds;

				cds.dwData = IntPtr.Zero;
				cds.lpData = strMsg;

				cds.cbData = System.Text.Encoding.Default.GetBytes(strMsg).Length + 1;

				int fromWindowHandler = 0;
				SendMessage(hwnd, WM_COPYDATA, fromWindowHandler, ref  cds);

			}
		}
		
		[StructLayout(LayoutKind.Sequential)]
		public struct CopyDataStruct {
			public IntPtr dwData;
			public int cbData;

			[MarshalAs(UnmanagedType.LPStr)]
			public string lpData;
		}
	}
}
